import { EventEmitter, Component, ChangeDetectionStrategy, ElementRef, NgZone, Renderer2, KeyValueDiffers, Input, Output, ViewChild, HostBinding, NgModule } from '@angular/core';
import { fromTextArea } from 'codemirror';
import { filter, take } from 'rxjs/operators';
import { CommonModule } from '@angular/common';

/**
 * @fileoverview added by tsickle
 * Generated from: codemirror.component.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CodeMirrorComponent {
    /**
     * @param {?} elementRef
     * @param {?} ngZone
     * @param {?} renderer
     * @param {?} _differs
     * @param {?} _ngZone
     */
    constructor(elementRef, ngZone, renderer, _differs, _ngZone) {
        this.elementRef = elementRef;
        this.ngZone = ngZone;
        this.renderer = renderer;
        this._differs = _differs;
        this._ngZone = _ngZone;
        this.autoMaxHeight = 0;
        this.codeChange = new EventEmitter();
        this.focusChange = new EventEmitter();
        this.codemirrorClassName = true;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        if (!this._differ && this.options) {
            this._differ = this._differs.find(this.options).create();
        }
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        if (this.elementRef.nativeElement.offsetWidth > 0) {
            this.initCodemirror();
            if (this.autoMaxHeight > 0) {
                this.renderer.setStyle(this.elementRef.nativeElement, "maxHeight", `${this.autoMaxHeight}px`);
            }
        }
        else {
            this.ngZone.onStable
                .asObservable()
                .pipe(filter((/**
             * @return {?}
             */
            () => {
                return this.elementRef.nativeElement.offsetWidth > 0;
            })), take(1))
                .subscribe((/**
             * @return {?}
             */
            () => {
                this.initCodemirror();
            }));
        }
    }
    /**
     * @param {?} simpleChanges
     * @return {?}
     */
    ngOnChanges(simpleChanges) {
        /** @type {?} */
        const optionsChange = simpleChanges.options;
        if (optionsChange && !optionsChange.firstChange) {
            /** @type {?} */
            const changes = this._differ.diff(this.options);
            if (changes) {
                changes.forEachChangedItem((/**
                 * @param {?} option
                 * @return {?}
                 */
                (option) => this.setOptionIfChanged(option.key, option.currentValue)));
                changes.forEachAddedItem((/**
                 * @param {?} option
                 * @return {?}
                 */
                (option) => this.setOptionIfChanged(option.key, option.currentValue)));
                changes.forEachRemovedItem((/**
                 * @param {?} option
                 * @return {?}
                 */
                (option) => this.setOptionIfChanged(option.key, option.currentValue)));
            }
        }
    }
    /**
     * @return {?}
     */
    initCodemirror() {
        this.ngZone.runOutsideAngular((/**
         * @return {?}
         */
        () => {
            if (!this.options) {
                throw new Error("options is required");
            }
            this.editor = fromTextArea(this.textAreaRef.nativeElement, this.options);
            this.editor.setValue(this.code ? this.code : "");
            this.editor.on("focus", (/**
             * @return {?}
             */
            () => this.ngZone.run((/**
             * @return {?}
             */
            () => this.focusChange.emit(true)))));
            this.editor.on("blur", (/**
             * @return {?}
             */
            () => {
                this.ngZone.run((/**
                 * @return {?}
                 */
                () => this.focusChange.emit(false)));
                this.editor.setCursor(0, null, { scroll: false });
            }));
            this.editor.on("change", (/**
             * @param {?} cm
             * @param {?} change
             * @return {?}
             */
            (cm, change) => {
                if (change.origin !== "setValue") {
                    this.code = cm.getValue();
                    this.codeChange.emit(this.code);
                }
            }));
            if (this.autoMaxHeight > 0) {
                this.initMaxHeight();
            }
        }));
    }
    /**
     * @private
     * @return {?}
     */
    initMaxHeight() {
        /** @type {?} */
        const codeWrapElement = (/** @type {?} */ (((/** @type {?} */ (this.elementRef
            .nativeElement))).querySelector(".CodeMirror-code")));
        this.applyEditorHeight(codeWrapElement);
        /** @type {?} */
        const codemirrorContentObserver = new MutationObserver((/**
         * @return {?}
         */
        () => {
            this.applyEditorHeight(codeWrapElement);
        }));
        codemirrorContentObserver.observe(codeWrapElement, {
            childList: true,
        });
    }
    /**
     * @private
     * @param {?} code
     * @return {?}
     */
    applyEditorHeight(code) {
        if (code.offsetHeight >= this.autoMaxHeight) {
            /** @type {?} */
            const wrapHeight = (/** @type {?} */ (((/** @type {?} */ (this.elementRef
                .nativeElement))).querySelector(".CodeMirror")));
            if (wrapHeight.offsetHeight >= this.autoMaxHeight) {
                this.editor.setSize("100%", `${this.autoMaxHeight}px`);
            }
        }
        else {
            this.editor.setSize("100%", `auto`);
        }
    }
    /**
     * @param {?} optionName
     * @param {?} newValue
     * @return {?}
     */
    setOptionIfChanged(optionName, newValue) {
        if (!this.editor) {
            return;
        }
        // cast to any to handle strictly typed option names
        // could possibly import settings strings available in the future
        this.editor.setOption((/** @type {?} */ (optionName)), newValue);
    }
    /**
     * @return {?}
     */
    ngOnDestroy() { }
}
CodeMirrorComponent.decorators = [
    { type: Component, args: [{
                selector: "ng-code-mirror, [ngCodeMirror]",
                template: `<textarea #textAreaRef></textarea>`,
                preserveWhitespaces: false,
                changeDetection: ChangeDetectionStrategy.OnPush
            }] }
];
/** @nocollapse */
CodeMirrorComponent.ctorParameters = () => [
    { type: ElementRef },
    { type: NgZone },
    { type: Renderer2 },
    { type: KeyValueDiffers },
    { type: NgZone }
];
CodeMirrorComponent.propDecorators = {
    code: [{ type: Input }],
    autoMaxHeight: [{ type: Input }],
    options: [{ type: Input }],
    codeChange: [{ type: Output }],
    focusChange: [{ type: Output }],
    textAreaRef: [{ type: ViewChild, args: ["textAreaRef", { read: ElementRef, static: true },] }],
    codemirrorClassName: [{ type: HostBinding, args: ["class.ng-codemirror",] }]
};
if (false) {
    /** @type {?} */
    CodeMirrorComponent.prototype.code;
    /** @type {?} */
    CodeMirrorComponent.prototype.autoMaxHeight;
    /** @type {?} */
    CodeMirrorComponent.prototype.options;
    /** @type {?} */
    CodeMirrorComponent.prototype.codeChange;
    /** @type {?} */
    CodeMirrorComponent.prototype.focusChange;
    /** @type {?} */
    CodeMirrorComponent.prototype.textAreaRef;
    /** @type {?} */
    CodeMirrorComponent.prototype.codemirrorClassName;
    /** @type {?} */
    CodeMirrorComponent.prototype.editor;
    /**
     * @type {?}
     * @private
     */
    CodeMirrorComponent.prototype._differ;
    /**
     * @type {?}
     * @private
     */
    CodeMirrorComponent.prototype.elementRef;
    /**
     * @type {?}
     * @private
     */
    CodeMirrorComponent.prototype.ngZone;
    /**
     * @type {?}
     * @private
     */
    CodeMirrorComponent.prototype.renderer;
    /**
     * @type {?}
     * @private
     */
    CodeMirrorComponent.prototype._differs;
    /**
     * @type {?}
     * @private
     */
    CodeMirrorComponent.prototype._ngZone;
}

/**
 * @fileoverview added by tsickle
 * Generated from: codemirror.module.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CodemirrorModule {
}
CodemirrorModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule],
                declarations: [CodeMirrorComponent],
                entryComponents: [CodeMirrorComponent],
                exports: [CodeMirrorComponent]
            },] }
];

/**
 * @fileoverview added by tsickle
 * Generated from: public-api.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * Generated from: ng-codemirror.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { CodeMirrorComponent, CodemirrorModule };
//# sourceMappingURL=ng-codemirror.js.map
