import { EventEmitter, Component, ChangeDetectionStrategy, ElementRef, NgZone, Renderer2, KeyValueDiffers, Input, Output, ViewChild, HostBinding, NgModule } from '@angular/core';
import { fromTextArea } from 'codemirror';
import { filter, take } from 'rxjs/operators';
import { CommonModule } from '@angular/common';

/**
 * @fileoverview added by tsickle
 * Generated from: codemirror.component.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var CodeMirrorComponent = /** @class */ (function () {
    function CodeMirrorComponent(elementRef, ngZone, renderer, _differs, _ngZone) {
        this.elementRef = elementRef;
        this.ngZone = ngZone;
        this.renderer = renderer;
        this._differs = _differs;
        this._ngZone = _ngZone;
        this.autoMaxHeight = 0;
        this.codeChange = new EventEmitter();
        this.focusChange = new EventEmitter();
        this.codemirrorClassName = true;
    }
    /**
     * @return {?}
     */
    CodeMirrorComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        if (!this._differ && this.options) {
            this._differ = this._differs.find(this.options).create();
        }
    };
    /**
     * @return {?}
     */
    CodeMirrorComponent.prototype.ngAfterViewInit = /**
     * @return {?}
     */
    function () {
        var _this = this;
        if (this.elementRef.nativeElement.offsetWidth > 0) {
            this.initCodemirror();
            if (this.autoMaxHeight > 0) {
                this.renderer.setStyle(this.elementRef.nativeElement, "maxHeight", this.autoMaxHeight + "px");
            }
        }
        else {
            this.ngZone.onStable
                .asObservable()
                .pipe(filter((/**
             * @return {?}
             */
            function () {
                return _this.elementRef.nativeElement.offsetWidth > 0;
            })), take(1))
                .subscribe((/**
             * @return {?}
             */
            function () {
                _this.initCodemirror();
            }));
        }
    };
    /**
     * @param {?} simpleChanges
     * @return {?}
     */
    CodeMirrorComponent.prototype.ngOnChanges = /**
     * @param {?} simpleChanges
     * @return {?}
     */
    function (simpleChanges) {
        var _this = this;
        /** @type {?} */
        var optionsChange = simpleChanges.options;
        if (optionsChange && !optionsChange.firstChange) {
            /** @type {?} */
            var changes = this._differ.diff(this.options);
            if (changes) {
                changes.forEachChangedItem((/**
                 * @param {?} option
                 * @return {?}
                 */
                function (option) {
                    return _this.setOptionIfChanged(option.key, option.currentValue);
                }));
                changes.forEachAddedItem((/**
                 * @param {?} option
                 * @return {?}
                 */
                function (option) {
                    return _this.setOptionIfChanged(option.key, option.currentValue);
                }));
                changes.forEachRemovedItem((/**
                 * @param {?} option
                 * @return {?}
                 */
                function (option) {
                    return _this.setOptionIfChanged(option.key, option.currentValue);
                }));
            }
        }
    };
    /**
     * @return {?}
     */
    CodeMirrorComponent.prototype.initCodemirror = /**
     * @return {?}
     */
    function () {
        var _this = this;
        this.ngZone.runOutsideAngular((/**
         * @return {?}
         */
        function () {
            if (!_this.options) {
                throw new Error("options is required");
            }
            _this.editor = fromTextArea(_this.textAreaRef.nativeElement, _this.options);
            _this.editor.setValue(_this.code ? _this.code : "");
            _this.editor.on("focus", (/**
             * @return {?}
             */
            function () {
                return _this.ngZone.run((/**
                 * @return {?}
                 */
                function () { return _this.focusChange.emit(true); }));
            }));
            _this.editor.on("blur", (/**
             * @return {?}
             */
            function () {
                _this.ngZone.run((/**
                 * @return {?}
                 */
                function () { return _this.focusChange.emit(false); }));
                _this.editor.setCursor(0, null, { scroll: false });
            }));
            _this.editor.on("change", (/**
             * @param {?} cm
             * @param {?} change
             * @return {?}
             */
            function (cm, change) {
                if (change.origin !== "setValue") {
                    _this.code = cm.getValue();
                    _this.codeChange.emit(_this.code);
                }
            }));
            if (_this.autoMaxHeight > 0) {
                _this.initMaxHeight();
            }
        }));
    };
    /**
     * @private
     * @return {?}
     */
    CodeMirrorComponent.prototype.initMaxHeight = /**
     * @private
     * @return {?}
     */
    function () {
        var _this = this;
        /** @type {?} */
        var codeWrapElement = (/** @type {?} */ (((/** @type {?} */ (this.elementRef
            .nativeElement))).querySelector(".CodeMirror-code")));
        this.applyEditorHeight(codeWrapElement);
        /** @type {?} */
        var codemirrorContentObserver = new MutationObserver((/**
         * @return {?}
         */
        function () {
            _this.applyEditorHeight(codeWrapElement);
        }));
        codemirrorContentObserver.observe(codeWrapElement, {
            childList: true,
        });
    };
    /**
     * @private
     * @param {?} code
     * @return {?}
     */
    CodeMirrorComponent.prototype.applyEditorHeight = /**
     * @private
     * @param {?} code
     * @return {?}
     */
    function (code) {
        if (code.offsetHeight >= this.autoMaxHeight) {
            /** @type {?} */
            var wrapHeight = (/** @type {?} */ (((/** @type {?} */ (this.elementRef
                .nativeElement))).querySelector(".CodeMirror")));
            if (wrapHeight.offsetHeight >= this.autoMaxHeight) {
                this.editor.setSize("100%", this.autoMaxHeight + "px");
            }
        }
        else {
            this.editor.setSize("100%", "auto");
        }
    };
    /**
     * @param {?} optionName
     * @param {?} newValue
     * @return {?}
     */
    CodeMirrorComponent.prototype.setOptionIfChanged = /**
     * @param {?} optionName
     * @param {?} newValue
     * @return {?}
     */
    function (optionName, newValue) {
        if (!this.editor) {
            return;
        }
        // cast to any to handle strictly typed option names
        // could possibly import settings strings available in the future
        this.editor.setOption((/** @type {?} */ (optionName)), newValue);
    };
    /**
     * @return {?}
     */
    CodeMirrorComponent.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () { };
    CodeMirrorComponent.decorators = [
        { type: Component, args: [{
                    selector: "ng-code-mirror, [ngCodeMirror]",
                    template: "<textarea #textAreaRef></textarea>",
                    preserveWhitespaces: false,
                    changeDetection: ChangeDetectionStrategy.OnPush
                }] }
    ];
    /** @nocollapse */
    CodeMirrorComponent.ctorParameters = function () { return [
        { type: ElementRef },
        { type: NgZone },
        { type: Renderer2 },
        { type: KeyValueDiffers },
        { type: NgZone }
    ]; };
    CodeMirrorComponent.propDecorators = {
        code: [{ type: Input }],
        autoMaxHeight: [{ type: Input }],
        options: [{ type: Input }],
        codeChange: [{ type: Output }],
        focusChange: [{ type: Output }],
        textAreaRef: [{ type: ViewChild, args: ["textAreaRef", { read: ElementRef, static: true },] }],
        codemirrorClassName: [{ type: HostBinding, args: ["class.ng-codemirror",] }]
    };
    return CodeMirrorComponent;
}());
if (false) {
    /** @type {?} */
    CodeMirrorComponent.prototype.code;
    /** @type {?} */
    CodeMirrorComponent.prototype.autoMaxHeight;
    /** @type {?} */
    CodeMirrorComponent.prototype.options;
    /** @type {?} */
    CodeMirrorComponent.prototype.codeChange;
    /** @type {?} */
    CodeMirrorComponent.prototype.focusChange;
    /** @type {?} */
    CodeMirrorComponent.prototype.textAreaRef;
    /** @type {?} */
    CodeMirrorComponent.prototype.codemirrorClassName;
    /** @type {?} */
    CodeMirrorComponent.prototype.editor;
    /**
     * @type {?}
     * @private
     */
    CodeMirrorComponent.prototype._differ;
    /**
     * @type {?}
     * @private
     */
    CodeMirrorComponent.prototype.elementRef;
    /**
     * @type {?}
     * @private
     */
    CodeMirrorComponent.prototype.ngZone;
    /**
     * @type {?}
     * @private
     */
    CodeMirrorComponent.prototype.renderer;
    /**
     * @type {?}
     * @private
     */
    CodeMirrorComponent.prototype._differs;
    /**
     * @type {?}
     * @private
     */
    CodeMirrorComponent.prototype._ngZone;
}

/**
 * @fileoverview added by tsickle
 * Generated from: codemirror.module.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var CodemirrorModule = /** @class */ (function () {
    function CodemirrorModule() {
    }
    CodemirrorModule.decorators = [
        { type: NgModule, args: [{
                    imports: [CommonModule],
                    declarations: [CodeMirrorComponent],
                    entryComponents: [CodeMirrorComponent],
                    exports: [CodeMirrorComponent]
                },] }
    ];
    return CodemirrorModule;
}());

/**
 * @fileoverview added by tsickle
 * Generated from: public-api.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * Generated from: ng-codemirror.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { CodeMirrorComponent, CodemirrorModule };
//# sourceMappingURL=ng-codemirror.js.map
